#!/usr/bin/perl

#    PERLFECT DAILY STATS version 1.4
#    (c) 1998 Perlfect Solutions  
#    http://perlfect.com
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#------------------------- Configuration --------------------------------#

# The full path to your access log file
my $ACCESS_LOG = '/home/httpd/access-log';

# The full path to the directory where the statistics will be kept
# This directory should be made publically accessible and lie within you
# www root tree in order for the statistics to be viewable via the web
my $DATA_DIR = '/home/httpd/html/stats/';

# The url corresponding to the above directory
my $STATS_URL = 'http://localhost/stats/';

# Your site's domain name. (without the www prefix, just the domain name)
# This will be used to identify internal referers and ignore them in case
# you choose to. It will also be used to mark the title of the reports
# so it is a good idea to set it anyway.
my $DOMAIN = 'perlfect.com';

# The url corresponding to your HTTP server's document root
my $BASE_URL = 'http://localhost/';

# Set this to 1 if you want to ignore internal referers (links from within
# your site) or 0 to include them in the report.
my $IGNORE_INTERNAL = 1;

#-------------- No User-Serviceable Parts Below This Line ---------------#


sub parse_log;
sub report;
sub get_keywords;

###################################################################### constants

my $PROGRAM_VERSION = '1.4';
my $PROGRAM_NAME = 'PERLFECT DAILY STATS';
my $PROGRAM_COPYRIGHT = '(C)1999 Perlfect Solutions';
my %MONTHS = (
	      Jan => '1',
	      Feb => '2',
	      Mar => '3',
	      Apr => '4',
	      May => '5',
	      Jun => '6',
	      Jul => '7',
	      Aug => '8',
	      Sep => '9',
	      Oct => '10',
	      Nov => '11',
	      Dec => '12'
	     );



my %LOG = ();

$|=1;
print "$PROGRAM_NAME v$PROGRAM_VERSION\n";
print "$PROGRAM_COPYRIGHT\n";

my @ltime = localtime(time);
my ($this_day, $this_month, $this_year) = ($ltime[3], $ltime[4]+1, $ltime[5]);
if(length("$this_year")==2)
  { # heh... we're y2k compliant, too
    ($this_year>70 and $this_year+=1900) or $this_year+=2000;
  }
print "The date is $this_day/$this_month/$this_year.\n";

my ($last_day, $last_month, $last_year) = (1, 1, 70);
if(open(LASTRUN, "$DATA_DIR/last_run"))
  {
    my $lastrun = <LASTRUN>;
    $lastrun=~m/(\d+) (\d+) (\d+)/;
    ($last_day, $last_month, $last_year) = ($1, $2, $3);
    close LASTRUN;
  }
print "$PROGRAM_NAME was last run on $last_day/$last_month/$last_year.\n";
my $arg = shift @ARGV;
print "Overwrite mode: Existing summary data will be overwriten.\n";
$last_year = 0;


print "Please wait while the HTTP log is being parsed...\n";
&parse_log;
print "Generating summary reports...\n";
my $date;
foreach $date (keys %LOG)
  {
    my $hlog = $LOG{$date};
    my ($view_html, $view_txt) = &report('Document Hits', $$hlog{view});
    my ($gway_html, $gway_txt) = &report('Gateways', $$hlog{gway});
    my ($ref_html, $ref_txt) = &report('Referers', $$hlog{ref});
    my ($refdom_html, $refdom_txt) = &report('Referring Domains', $$hlog{refdom});
    my ($refproto_html, $refproto_txt) = &report('Referer Protocols', $$hlog{refproto});
    my ($keywords_html, $keywords_txt) = &report('Search Keywords', $$hlog{keywords});

    my $html =
"
<html>
<title>Access Statistics for $this_day/$this_month/$this_year</title>
<body bgcolor=\"#ffffff\" text=\"#000000\" link=\"#003366\" vlink=\"#000000\">
<center>
<h1>
<font color=\"#006699\">
Access Statistics for $DOMAIN - $date
</font>
</h1>
$view_html
<br><br>
$gway_html
<br><br>
$ref_html
<br><br>
$refdom_html
<br><br>
$refproto_html
<br><br>
$keywords_html
<br><BR>
<hr>
<font size=-1>
Statistics generated automatically by $PROGRAM_NAME - (C)1999 <a href=\"http://perlfect.com\">PERLFECT SOLUTIONS</a>
</font>
</center>
</body>
</html>
";

    my $text =
"
\t\tACCESS STATISTICS FOR $DOMAIN - $date

$view_txt
$gway_txt
$ref_txt
$refdom_txt
$refproto_txt
$keywords_txt
";

    print "$date :";
    open(HTML, ">$DATA_DIR/$date.html") or die $!;
    print HTML $html;
    close HTML;
    print " HTML";

    open(TEXT, ">$DATA_DIR/$date.txt") or die $!;
    print TEXT $text;
    close TEXT;
    print " TEXT\n";

  }

if(open(LASTRUN, ">$DATA_DIR/last_run"))
  {
    print LASTRUN "$this_day $this_month $this_year";
  }

print "Done.\n";






######################################################################## subroutines


sub parse_log
  {
    open(LOG, "$ACCESS_LOG") or die "Cannot open access log: $!\n";
  RECORD:
    while(<LOG>)
      {
	m/^.*? .*? .*? \[(\d+)\/(\S+)\/(\d+).*?\] \".*? (.*?) .*?\" \d+ \d+( \"(.*?)\" \".*?\")?/
	  or next RECORD;
	my($day, $month, $year, $doc, $ref) = ($1, $2, $3, $4, '');
	my $ref = $6 if(defined($6));
	next RECORD if($year<$last_year);
	next RECORD if($year==$last_year and $month<$last_month);
	next RECORD if($year==$last_year and $month==$last_month and $day<$last_day);
	next RECORD if($year==$this_year and $month==$this_month and $day==$this_day);
	next RECORD unless($doc=~m/(\/|html?|pl|cgi|\?.*)$/);
	$doc=~s/\/(index|default)\.s?html?$/\//g;
	$month = $MONTHS{$month};
	map
	  {
	    $_='0'.$_ unless(length($_)==2);
	  } ($day, $month);
	my $date = $year.'_'.$month.'_'.$day;
	$LOG{$date} = { 
		       view=>{}, 
		       gway=>{}, 
		       ref=>{},
		       refdom=>{},
		       refproto=>{},
		       keywords=>{},
		      } unless(defined($LOG{$date}));
	my $hview = ${$LOG{$date}}{view};
	my $hgway = ${$LOG{$date}}{gway};
	my $href = ${$LOG{$date}}{ref};
	my $hrefdom = ${$LOG{$date}}{refdom};
	my $hrefproto = ${$LOG{$date}}{refproto};
	my $hkeywords = ${$LOG{$date}}{keywords};
	$$hview{$doc}++;
	$$hview{'_TOTAL_'}++;
	my $refdom = $ref;
	$refdom=~s/^(\S+):\/\/([A-Za-z\-\.\d]*).*$/$2/;
	my $refproto=$1;
	$refdom=~s/^.*\.(.*\..*)$/$1/;
	if ($ref eq '-' or length($ref) == 0)
	  {
	    $ref = 'BOOKMARKS/TYPE-IN';
	    $refdom = 'BOOKMARKS/TYPE-IN';
	    $refproto = 'BOOKMARKS/TYPE-IN';
	  }
	next RECORD if(
		       ($IGNORE_INTERNAL and $refdom=~m/$DOMAIN/)
		      );
	$$href{_TOTAL_}++;
	$$href{_DISTINCT_}++ unless($$href{$ref});
	$$href{$ref}++;
	$$hrefdom{_TOTAL_}++;
	$$hrefdom{_DISTINCT_}++ unless($$hrefdom{$refdom});
	$$hrefdom{$refdom}++;
	$$hrefproto{_TOTAL_}++;
	$$hrefproto{_DISTINCT_}++ unless($$hrefproto{$refproto});
	$$hrefproto{$refproto}++;
	next RECORD if($refdom=~m/$DOMAIN/);
	$$hgway{_TOTAL_}++;
	$$hgway{'_DISTINCT_'}++ unless($$hgway{$doc});
	$$hgway{$doc}++;
	foreach (get_keywords($ref, $refdom))
	  {
	    $$hkeywords{_TOTAL_}++;
	    $$hkeywords{_DISTINCT_}++ unless($$hkeywords{$_});
	    $$hkeywords{$_}++;
	  }
      }
    close LOG;
  }


sub report
  {
    my $report = shift;
    my $hr = shift;
    my $html = "<table width=\"760\" border=\"0\" cellspacing=\"0\" cellpadding=\"2\">\n";
    $html .= "<tr bgcolor=\"#006699\"><td colspan=\"4\">";
    $html .= "<font color=\"#ffffff\" size=+1><b>$report</font>\n";
    $html .= "<font color=\"#ffffff\" size=\"-1\"><br>$$hr{_TOTAL_} TOTAL";
    $html .= "/ $$hr{_DISTINCT_} DISTINCT\n" if($$hr{_DISTINCT_});
    $html .= "</font></b></td></tr>\n";
    my $txt = "$report\n----------\n";
    my $count = 1;
    my $k;
    my $color;
  DOCUMENT:
    foreach $k ( reverse sort { $$hr{$a} <=> $$hr{$b} } keys %$hr )
      {
	next DOCUMENT if($k eq '_TOTAL_' or $k eq '_DISTINCT_');
	if($count/2 == int($count/2))
	  {
	    $color = '#c0c0c0';
	  }
	else
	  {
	    $color = '#eeeeee';
	  }
	my $location = '';
	if($k=~m/^\//)
	  {
	    $location = "$BASE_URL$k";
	  }
	elsif($k=~m/^http:\/\//)
	  {
	    $location = $k;
	  }
	$k_trunc = substr($k, 0, 75);
	my $percent = int(($$hr{$k}/$$hr{'_TOTAL_'})*100) if($$hr{'_TOTAL_'});
	my $barlen = $percent*3+1;
	$html .= "<tr bgcolor=\"$color\"><td align=\"right\"><font size=\"-1\">$count</font></td>\n";
	$html .= "<td><font size=\"-1\">$$hr{$k} [$percent%]</td></font>\n";
	$html .= "<td align=\"left\"><font size=\"-1\">";
	$html .= "<a href=\"$location\">" if($location);
	$html .= "$k_trunc";
	$html .= "</a>" if($location);
	$html .= "</font></td>\n";
	$html .= "<td align=\"left\" valign=\"center\">";
	$html .= "<img src=\"$STATS_URL/bar.gif\" height=\"6\" width=\"$barlen\"></td></tr>\n";
	$txt .= "$$hr{$k}\t[$percent%]\t$k\n";
	$count++;
      }
    $html .= "</table>";
    $txt .= "TOTAL    : $$hr{_TOTAL_}\n";
    $txt .= "DISTINCT : $$hr{_DISTINCT_}\n" if($$hr{_DISTINCT_});
    $txt .= "\n\n";
    return ($html, $txt);
  }



sub get_keywords
  {
    my $url = shift;
    my $refdom = shift;
    my @keywords = ();
    
    $url=~m/http:\/\/.*?\/.*?\?(.*)$/;
    my $q = $1;

    my $f1 = '';
    if($refdom=~m/altavista/)
      {
	$f1 = 'q';
      }
    elsif($refdom=~m/goto/)
      {
	$f1 = 'Keywords';
      }
    elsif($refdom=~m/hotbot|search\.msn/)
      {
	$f1 = 'MT';
      }
    elsif($refdom=~m/search\.go2net/)
      {
	$f1 = 'general';
      }
    elsif($refdom=~m/snap/)
      {
	$f1 = 'keyword';
      }
    elsif($refdom=~m/ink\.yahoo/)
      {
	$f1 = 'p';
      }
    elsif($refdom=~m/dogpile|evreka|euroseek/)
      {
	$f1 = 'q';
      }
    elsif($refdom=~m/infoseek/)
      {
	$f1 = 'qt';
      }
    elsif($refdom=~m/anzwers|euroseek|lycos/)
      {
	$f1 = 'query';
      }
    elsif($refdom=~m/netfind\.aol/)
      {
	$f1 = 's';
      }
    elsif($refdom=~m/excite|excitesearch\.netscape/)
      {
	$f1 = 'search';
      }

    if($f1 && $q=~m/$f1=(.*?)\&/)
      {
	my $ks = $1;
	$ks =~ s/%([A-Za-z\d]{2})/pack("C", hex($1))/eg;
	while($ks=~s/([A-Za-z]+)//)
	  {
	    push @keywords, $1;
	  }
      }
    
    return @keywords;
  }
